"""Manage rich themes: implements ThemeManager class"""


import pathlib
from os import unlink
from os.path import exists
from typing import Dict, List, Optional

from rich.console import Console
from rich.table import Table

from .theme import SAMPLE_TEXT, Theme


class ThemeManager:
    """Manage rich themes"""

    def __init__(
        self,
        theme_dir: Optional[str] = None,
        themes: Optional[List[Theme]] = None,
    ):
        """Create ThemeManager instance

        Args:
            theme_dir (Optional[str]): directory containing themes
            themes (Optional[List[Theme]]): list of rich_theme_manager.Theme objects
        """
        self._theme_dir: Optional[pathlib.Path] = (
            pathlib.Path(theme_dir) if theme_dir else None
        )
        self._themes: Dict[str, Theme] = (
            {theme.name: theme for theme in themes} if themes else {}
        )

        if self._theme_dir is not None:
            for theme in self.themes:
                theme.path = str(self._theme_dir / f"{theme.name}.theme")
            self.load_themes()
            self.write_themes()

    @property
    def themes(self) -> List[Theme]:
        """Themes"""
        return list(self._themes.values())

    def add(self, theme: Theme, overwrite=False) -> None:
        """Add theme; if theme file doesn't exist, it will be written"""
        if self._theme_dir and not theme.path:
            theme.path = str(self._theme_dir / f"{theme.name}.theme")
        if self._theme_dir and overwrite or (theme.path and not exists(theme.path)):
            theme.save(overwrite=overwrite)
        self._themes[theme.name] = theme

    def remove(self, theme: Theme) -> None:
        """Remove theme; if theme file exists, it will be deleted"""
        if theme.path and exists(theme.path):
            unlink(theme.path)
        del self._themes[theme.name]

    def get(self, theme_name: str) -> Theme:
        """Get theme by name"""
        for theme in self.themes:
            if theme.name == theme_name:
                return theme
        raise ValueError(f"Theme {theme_name} not found")

    def load_themes(self, theme_dir: Optional[str] = None) -> None:
        """Load themes"""
        if theme_dir:
            # load themes from user specified theme directory instead of self._theme_dir
            _theme_dir = pathlib.Path(theme_dir)
            for path in _theme_dir.glob("*.theme"):
                theme = Theme.read(str(path))
                self._themes[theme.name] = theme
        elif self._theme_dir:
            for path in self._theme_dir.glob("*.theme"):
                theme = Theme.read(str(path))
                self._themes[theme.name] = theme
        else:
            raise ValueError("No theme directory specified")

    def write_themes(self, overwrite: bool = False) -> None:
        """Write themes"""
        for theme in self.themes:
            if not theme.path:
                raise ValueError(f"Theme {theme.name} has no path")
            if not exists(theme.path) or overwrite:
                theme.save(overwrite=overwrite)

    def list_themes(
        self,
        show_path: bool = True,
        theme_names: Optional[List[str]] = None,
        console: Optional[Console] = None,
    ) -> None:
        """List themes

        Args:
            show_path (bool): show theme file path
            theme_names (Optional[List[str]]): list of theme names to show (default is all themes)
            console: Optional[Console]: rich.console.Console instance to use for printing
        """
        table = Table(show_header=True, show_lines=False, box=None)
        table.add_column("Theme")
        table.add_column("Description")
        table.add_column("Tags")
        if show_path:
            table.add_column("Path")
        for theme in self.themes:
            if theme_names and theme.name not in theme_names:
                continue
            row = [
                theme.name,
                theme.description,
                str(", ".join(theme.tags)),
            ]
            if show_path:
                row.append(theme.path or "")
            table.add_row(*row)
        console = console or Console()
        console.print(table)

    @classmethod
    def preview_theme(
        cls,
        theme: Theme,
        sample_text: Optional[str] = None,
        show_path: bool = True,
        console: Optional[Console] = None,
    ) -> None:
        """Preview a theme to the console

        Args:
            theme (Theme): theme to preview
            sample_text (Optional[str]): sample text to use for preview (default is rich_theme_manager.theme.SAMPLE_TEXT)
            show_path (bool): show theme file path
            console: Optional[Console]: rich.console.Console instance to use for printing
        """
        console = console or Console()
        console.print(
            *list(
                theme._preview(
                    sample_text=sample_text or SAMPLE_TEXT, show_path=show_path
                )
            )
        )



"""Managed Theme class for use with ThemeManager; subclass of rich.theme.Theme"""

import configparser
from io import StringIO
from os.path import exists
from typing import IO, Dict, List, Mapping, Optional

import rich.theme
from rich.color import Color
from rich.console import Console, ConsoleOptions, RenderResult
from rich.panel import Panel
from rich.style import Style, StyleType
from rich.table import Table, box
from rich.text import Text

SAMPLE_TEXT = "The quick brown fox..."


class Theme(rich.theme.Theme):
    def __init__(
        self,
        name: str,
        description: Optional[str] = None,
        styles: Optional[Mapping[str, StyleType]] = None,
        inherit: bool = True,
        tags: Optional[List[str]] = None,
        path: Optional[str] = None,
    ):
        """Create a Theme instance

        Args:
            name (str): Name of the theme
            description (Optional[str]): Description of the theme
            styles (Optional[Mapping[str, StyleType]]): Styles to use in the theme
            inherit (bool): Whether or not to inherit from the default theme
            tags (Optional[List[str]]): Tags to use for filtering themes
            path (Optional[str]): Path to save the theme to
        """
        self._rtm_name: str = name
        self._rtm_description: str = description or ""
        self._rtm_styles: List[str] = list(styles.keys() if styles else [])
        self._rtm_inherit: bool = inherit
        self._rtm_tags: List[str] = tags or []
        self._rtm_path: Optional[str] = path
        super().__init__(styles=styles, inherit=inherit)

    @property
    def name(self) -> str:
        return self._rtm_name

    @property
    def description(self) -> str:
        return self._rtm_description

    @property
    def tags(self) -> List[str]:
        return self._rtm_tags

    @property
    def inherit(self) -> bool:
        return self._rtm_inherit

    @property
    def style_names(self) -> List[str]:
        return self._rtm_styles

    @property
    def path(self) -> Optional[str]:
        return self._rtm_path

    @path.setter
    def path(self, path: str):
        self._rtm_path = path

    @property
    def config(self) -> str:
        """Get contents of a config file for this theme."""
        metadata: Dict = {
            "name": self.name,
            "description": self.description,
            "tags": ", ".join(self.tags) if self.tags else "",
            "inherit": self.inherit,
        }
        config = configparser.ConfigParser()
        config.add_section("metadata")
        for key, value in metadata.items():
            config.set("metadata", key, str(value))
        strio = StringIO()
        config.write(strio)

        styles: str = "[styles]\n" + "\n".join(
            f"{name} = {style}"
            for name, style in sorted(self.styles.items())
            if name in self.style_names
        )

        return strio.getvalue() + styles + "\n"

    def to_file(self, path: str) -> None:
        """Write this theme to a config file."""
        with open(path, "w") as f:
            f.write(self.config)

    def save(self, overwrite=False) -> None:
        """Save this theme to its path."""
        if not self.path:
            raise ValueError(f"No path for theme {self.name}")
        if not overwrite and exists(self.path):
            raise FileExistsError(f"Theme {self.name} already exists at {self.path}")
        self.to_file(self.path)

    def load(self) -> None:
        """Load this theme from its path overwriting any theme data in memory."""
        if not self.path:
            raise FileNotFoundError(f"No path for theme {self.name}")
        if not exists(self.path):
            raise FileNotFoundError(f"Theme {self.name} does not exist at {self.path}")
        new_theme = Theme.read(self.path)
        self._rtm_name = new_theme.name
        self._rtm_description = new_theme.description
        self._rtm_styles = new_theme.style_names.copy()
        self._rtm_inherit = new_theme.inherit
        self._rtm_tags = new_theme.tags.copy()
        self._rtm_path = new_theme.path

    def _preview(
        self, sample_text: Optional[str] = None, show_path: bool = True
    ) -> RenderResult:
        """Preview a theme for printing to console"""
        title: str = f"Theme: {self.name}"
        if show_path and self.path:
            title += f" - {self.path}"

        table = Table(
            title=title,
            title_justify="center",
            show_header=True,
            show_lines=True,
            header_style="bold",
            box=box.SQUARE,
        )

        sample_text = sample_text or SAMPLE_TEXT

        for column in [
            "style",
            "color",
            "color",
            "bgcolor",
            "bgcolor",
            "attributes",
            "example",
        ]:
            table.add_column(column)

        for style_name in self.style_names:
            style = self.styles.get(style_name)
            if not style:
                continue
            color = (style.color.name or style.color.rgb) if style.color else "None"
            bgcolor = (
                (style.bgcolor.name or style.bgcolor.rgb) if style.bgcolor else "None"
            )

            attributes = attribute_str(style)

            table.add_row(
                style_name,
                str(color),
                color_bar(5, style.color) if style.color else " " * 5,
                str(bgcolor),
                color_bar(5, style.bgcolor) if style.bgcolor else " " * 5,
                attributes,
                Text(sample_text, style=style),
            )

        yield table

        legend_table = Table(
            show_header=False,
            show_lines=False,
            box=None,
        )
        legend_table.add_row(
            (
                f"{_bold('b')}: bold, "
                f"{_bold('d')}: dim, "
                f"{_bold('i')}: italic, "
                f"{_bold('u')}: underline, "
                f"{_bold('U')}: double underline, "
                f"{_bold('B')}: blink, "
                f"{_bold('2')}: blink2"
            )
        )
        legend_table.add_row(
            (
                f"{_bold('r')}: reverse, "
                f"{_bold('c')}: conceal, "
                f"{_bold('s')}: strike, "
                f"{_bold('f')}: frame, "
                f"{_bold('e')}: encircle, "
                f"{_bold('o')}: overline, "
                f"{_bold('L')}: Link"
            )
        )

        yield Panel(
            legend_table,
            box=box.SQUARE,
            title="attributes legend",
            title_align="left",
            expand=False,
        )

    def __eq__(self, other: object) -> bool:
        return (
            self.name == other.name
            and self.description == other.description
            and self.styles == other.styles
            and self.inherit == other.inherit
            and self.tags == other.tags
        ) if isinstance(other, Theme) else NotImplemented

    def __rich_console__(
        self, console: Console, options: ConsoleOptions
    ) -> RenderResult:
        yield from self._preview()

    @classmethod
    def from_file(
        cls, config_file: IO[str], source: Optional[str] = None, inherit: bool = True
    ) -> "Theme":
        """Load a theme from a text mode file.
        Args:
            config_file (IO[str]): An open conf file.
            source (str, optional): The filename of the open file. Defaults to None.
            inherit (bool, optional): Inherit default styles. Defaults to True.
        Returns:
            Theme: A New theme instance.
        """
        config = configparser.ConfigParser()
        config.read_file(config_file, source=source)
        styles: Dict = {
            name: Style.parse(value) for name, value in config.items("styles")
        }
        metadata: Dict = dict(config.items("metadata"))
        inherit = inherit or metadata.get("inherit", False)
        tags: List[str] = (
            metadata.get("tags", "").split(",") if metadata.get("tags") else []
        )
        return Theme(
            name=metadata["name"],
            description=metadata.get("description") or "",
            tags=tags,
            styles=styles,
            inherit=inherit,
            path=source,
        )

    @classmethod
    def read(cls, path: str, inherit: bool = True) -> "Theme":
        """Read a theme from a path.
        Args:
            path (str): Path to a config file readable by Python configparser module.
            inherit (bool, optional): Inherit default styles. Defaults to True.
        Returns:
            Theme: A new theme instance.
        """
        with open(path, "rt") as config_file:
            return cls.from_file(config_file, source=path, inherit=inherit)


def _bold(text: str) -> str:
    return f"[bold]{text}[/]"


def attribute_str(style: Style) -> str:
    """Return a string representing all attributes of a style"""
    attributes = "" + (_bold("b") if style.bold else "-")
    attributes += _bold("d") if style.dim else "-"
    attributes += _bold("i") if style.italic else "-"
    attributes += _bold("u") if style.underline else "-"
    attributes += _bold("U") if style.underline2 else "-"
    attributes += _bold("B") if style.blink else "-"
    attributes += _bold("2") if style.blink2 else "-"
    attributes += _bold("r") if style.reverse else "-"
    attributes += _bold("c") if style.conceal else "-"
    attributes += _bold("s") if style.strike else "-"
    attributes += _bold("f") if style.frame else "-"
    attributes += _bold("e") if style.encircle else "-"
    attributes += _bold("o") if style.overline else "-"
    attributes += _bold("L") if style.link else "-"
    return attributes


def color_bar(length: int, color: Color) -> str:
    """Create a color bar."""
    bar = "█" * length
    return Text(bar, style=Style(color=color))



"""Theme Manager for rich Themes"""
import importlib.metadata

from .manager import ThemeManager
from .theme import Theme

__version__ = importlib.metadata.version("rich_theme_manager")

__all__ = ["Theme", "ThemeManager"]



"""Example CLI usage of rich_theme_manager.
Uses argparse for no dependencies but should be easily adapatable to click or typer
"""

import argparse
import os
import sys
import tempfile
from textwrap import dedent

from rich.console import Console
from rich.style import Style

# import Theme from rich_theme_manager
from rich_theme_manager import Theme, ThemeManager

# Sample themes for use in the CLI
THEMES = [
    Theme(
        name="dark",
        description="Dark mode theme",
        tags=["dark"],
        styles={
            "hidden": Style(color="#383b3d", dim=True),
            "error": Style(color="rgb(255,85,85)", bold=True),
            "filename": Style(color="rgb(189,147,249)", bold=True),
            "filepath": Style(color="rgb(80,250,123)", bold=True),
            "highlight": Style(color="#000000", bgcolor="#d73a49", bold=True),
            "num": Style(color="rgb(139,233,253)", bold=True),
            "time": Style(color="rgb(139,233,253)", bold=True),
            "warning": Style(color="rgb(241,250,140)", bold=True),
        },
    ),
    Theme(
        name="light",
        description="Light mode theme",
        styles={
            "hidden": Style(color="#383b3d", dim=True),
            "error": Style(color="#b31d28", bold=True, underline=True, italic=True),
            "filename": Style(color="#6f42c1", bold=True),
            "filepath": Style(color="#22863a", bold=True),
            "highlight": Style(color="#ffffff", bgcolor="#d73a49", bold=True),
            "num": Style(color="#005cc5", bold=True),
            "time": Style(color="#032f62", bold=True),
            "warning": Style(color="#e36209", bold=True, underline=True, italic=True),
        },
    ),
    Theme(
        name="mono",
        description="Monochromatic theme",
        tags=["mono", "colorblind"],
        styles={
            "hidden": "dim",
            "error": "reverse italic",
            "filename": "bold",
            "filepath": "bold underline",
            "highlight": "reverse italic",
            "num": "bold",
            "time": "bold",
            "warning": "bold italic",
        },
    ),
    Theme(
        name="plain",
        description="Plain theme with no colors",
        tags=["colorblind"],
        styles={
            "hidden": "",
            "error": "",
            "filename": "",
            "filepath": "",
            "highlight": "",
            "num": "",
            "time": "",
            "warning": "",
        },
    ),
]


def get_theme_dir() -> str:
    """Get the theme directory"""
    # with click, this would be a good place to use get_app_dir
    theme_dir = os.path.join(tempfile.gettempdir(), ".rich_theme_manager")
    if not os.path.exists(theme_dir):
        os.mkdir(theme_dir)
    return theme_dir


def validate_theme_name(name: str) -> None:
    """Check if name is a valid theme name"""
    valid_themes = [theme.name for theme in THEMES]
    if name not in valid_themes:
        print(
            f"'{name}' is not a valid theme name. Valid themes are: {valid_themes}",
            file=sys.stderr,
        )
        sys.exit(1)


def print_example_output(theme: Theme) -> None:
    """Print example output using theme"""
    example_text = f"""
    The following shows examples of how to use custom rich styles with your text.

    This is an example filepath: [filepath]{sys.argv[0]}[/]
    This is an example filename: [filename]{os.path.split(sys.argv[0])[1]}[/]
    This is an example of hidden filename: [hidden].zshrc[/]
    This is an example of a warning: [warning]I've giv'n her all she's got captain, an' I canna give her no more.[/]
    This is an example of an error: [error]I'm sorry, Dave. I'm afraid I can't do that.[/]
    This is an example of a highlight: [highlight]foo[/]
    This is an example of a number: [num]42[/]
    This is an example of a time: [time]12:34[/]
    """
    console = Console(theme=theme, highlight=False)
    console.print(dedent(example_text))


def cli():
    """Example CLI for use of rich_theme_manager"""
    # using argparse so there are no dependencies but dang,
    # does this make me appreciate click and typer better!
    parser = argparse.ArgumentParser(
        prog="rich_theme_manager", description="Example CLI usage of rich_theme_manager"
    )
    parser.add_argument(
        "--example",
        nargs="?",
        const="dark",
        help="Show example output for theme.",
    )
    parser.add_argument(
        "--list",
        help="List themes.",
        action="store_true",
    )
    parser.add_argument(
        "--preview",
        metavar="THEME",
        nargs=1,
        help="Preview theme.",
    )
    parser.add_argument(
        "--config",
        metavar="THEME",
        nargs=1,
        help="Print configuration for theme THEME.",
    )
    args = parser.parse_args()
    theme_manager = ThemeManager(theme_dir=get_theme_dir(), themes=THEMES)

    if not any([args.example, args.list, args.preview, args.config]):
        parser.print_help()
        return

    if args.example:
        validate_theme_name(args.example)
        theme = theme_manager.get(args.example)
        print_example_output(theme)
        return

    if args.list:
        theme_manager.list_themes()
        return

    if args.preview:
        validate_theme_name(args.preview[0])
        theme = theme_manager.get(args.preview[0])
        theme_manager.preview_theme(theme)
        return

    if args.config:
        theme = theme_manager.get(args.config[0])
        print(theme.config)
        return


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rich_theme_manager']

package_data = \
{'': ['*']}

install_requires = \
['rich>=11.0']

entry_points = \
{'console_scripts': ['rich_theme_manager = rich_theme_manager:main']}

setup_kwargs = {
    'name': 'rich-theme-manager',
    'version': '0.7.0',
    'description': 'Manage rich themes for CLI applications',
    'long_description': '# Rich Theme Manager\n\n## Description\n\nImplements a basic "theme manager" class for managing rich [Themes](https://rich.readthedocs.io/en/stable/style.html#style-themes) in your [rich](https://github.com/Textualize/rich) CLI application.\n\nThe rich package provides an easy way to define custom styles and themes for use with rich.  This package provides a simple way to manage the themes: e.g. to list, add, remove themes, for the user to preview themes, and to manage themes on disk.\n\n## What problem does this solve?\n\nConsider this scenario from the rich [documentation](https://rich.readthedocs.io/en/stable/style.html#style-themes):\n\nIf you re-use styles it can be a maintenance headache if you ever want to modify an attribute or color – you would have to change every line where the style is used. Rich provides a Theme class which you can use to define custom styles that you can refer to by name. That way you only need to update your styles in one place.\n\nStyle themes can make your code more semantic, for instance a style called "warning" better expresses intent that "italic magenta underline".\n\nTo use a style theme, construct a Theme instance and pass it to the Console constructor. Here’s an example:\n\n```python\nfrom rich.console import Console\nfrom rich.theme import Theme\ncustom_theme = Theme({\n    "info": "dim cyan",\n    "warning": "magenta",\n    "danger": "bold red"\n})\nconsole = Console(theme=custom_theme)\nconsole.print("This is information", style="info")\nconsole.print("[warning]The pod bay doors are locked[/warning]")\nconsole.print("Something terrible happened!", style="danger")\n```\n\nI highly recommend the use of Themes in your rich application instead of hard-coding colors and styles. However, there\'s still a problem of managing themes.  For example, letting the user of your application change the default theme styles you\'ve chosen.  What if they are color blind and can\'t distinguish the colors you\'ve selected? What if they\'re using a light terminal background and you\'ve selected colors best suited for a dark terminal?  \n\nThe rich `Theme` class provides everything you to manage these situations but doing so requires a fair amount of boiler plate code for each application. This package attempts to provide an easy solution for these scenarios with a minimal amount of code.  Instead of having to implement management of theme files and allowing the user to list or preview themes yourself, you can use the `ThemeManager` class to manage themes for you.\n\n## Synopsis\n\nUsing `rich_theme_manager` is easy and takes just a few lines of code.  Import `Theme` from `rich_theme_manager` instead of from `rich.theme` (`rich_theme_manager.Theme` subclasses `rich.theme.Theme`) then use `ThemeManager` to manage themes.  `ThemeManager` can be created with or without a `theme_dir` argument. If you don\'t provide `theme_dir`, `ThemeManager` will not manage themes on disk.  This may still be useful for allowing the user to list and preview themes.  If you do provide `theme_dir`, any default themes passed to `ThemeManager` will be written to `theme_dir` and any theme config files found in `theme_dir` will be loaded.\n\n```python\nfrom rich.console import Console\nfrom rich.style import Style\nimport pathlib\n\nfrom rich_theme_manager import Theme, ThemeManager\n\n\nTHEMES = [\n    Theme(\n        name="dark",\n        description="Dark mode theme",\n        tags=["dark"],\n        styles={\n            "info": "dim cyan",\n            "warning": "bold magenta",\n            "danger": "bold red",\n        },\n    ),\n    Theme(\n        name="light",\n        description="Light mode theme",\n        styles={\n            "info": Style(color="#22863a", bold=True),\n            "warning": Style(color="#032f62", bold=True),\n            "danger": Style(color="#b31d28", bold=True, underline=True, italic=True),\n        },\n    ),\n    Theme(\n        name="mono",\n        description="Monochromatic theme",\n        tags=["mono", "colorblind"],\n        styles={\n            "info": "italic",\n            "warning": "bold",\n            "danger": "reverse bold",\n        },\n    ),\n]\n\nif __name__ == "__main__":\n    # you can specify a config directory to save/load themes to/from\n    theme_dir = pathlib.Path("~/.rich_theme_manager/themes").expanduser()\n    theme_dir.expanduser().mkdir(parents=True, exist_ok=True)\n\n    theme_manager = ThemeManager(theme_dir=theme_dir, themes=THEMES)\n    theme_manager.list_themes()\n    print("\\n")\n\n    dark = theme_manager.get("dark")\n    theme_manager.preview_theme(dark)\n    console = Console(theme=dark)\n    print("\\n")\n\n    console.print("This is information", style="info")\n    console.print("[warning]The pod bay doors are locked[/warning]")\n    console.print("Something terrible happened!", style="danger")\n```\n\n![Example output](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/example1.png)\n\n## Example app\n\nA simple example app that demonstrates the ThemeManager class comes with rich_theme_manager in `__main__.py`:\n\n`python -m rich_theme_manager`:\n\n```text\nusage: rich_theme_manager [-h] [--example [EXAMPLE]] [--list] [--preview THEME] [--config THEME]\n\nExample CLI usage of rich_theme_manager\n\noptional arguments:\n  -h, --help           show this help message and exit\n  --example [EXAMPLE]  Show example output for theme.\n  --list               List themes.\n  --preview THEME      Preview theme.\n  --config THEME       Print configuration for theme THEME.\n```\n\n`python -m rich_theme_manager --list`:\n\n![Example --list output](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/list.png)\n\n`python -m rich_theme_manager --preview dark`:\n\n![Example --preview output](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/preview_dark.png)\n\n## Documentation\n\n### Theme class\n\n`rich_theme_manager.Theme` is a subclass of `rich.theme.Theme` and provides additional functionality for managing themes.\n\n```python\n    Theme(\n          name: str,\n          description: Optional[str] = None,\n          styles: Optional[Mapping[str, StyleType]] = None,\n          inherit: bool = True,\n          tags: Optional[List[str]] = None,\n          path: Optional[str] = None,\n    )\n```\n\nArguments\n\n* `name`: The name of the theme; required\n* `description`: An optional description of the theme.\n* `styles`: An optional mapping of style names to styles.\n* `inherit`: Whether the theme inherits from the default theme.\n* `tags`: An optional list of tags for the theme; useful for allowing user to filter themes.\n* `path`: The path to the theme file; in normal use this is not needed as `ThemeManager` will automatically create the theme file.\n\nProperties\n\n* `Theme().name`: The name of the theme\n* `Theme().description`: Description of the theme\n* `Theme().tags`: List of tags for the theme\n* `Theme().inherit`: bool indicating whether the theme inherits from the default theme\n* `Theme().style_names`: List of names for styles in the theme\n* `Theme().path`: The path to the theme file; (getter/setter)\n* `Theme().config`: Contents of a configuration file for the theme (same format as `rich.theme.Theme().config` but with an additional `[metadata]` section)\n\nMethods\n\n* `Theme().save()`: Save the theme to disk (to `Theme().path`)\n* `Theme().load()`: Load the theme from disk (from `Theme().path`)\n* `Theme().to_file(path: str)`: Save the theme to disk (to `path`)\n\nClass Methods:\n\n* `Theme.from_file(config_file: IO[str], source: Optional[str] = None, inherit: bool = True)` -> `Theme`: Load a theme from a text mode configuration file (in [configparser](https://docs.python.org/3/library/configparser.html) INI format).\n* `Theme.read(path: str, inherit: bool = True) -> Theme`: Load a theme from disk (from `path`)\n\nThe `.theme` INI file format looks like this:\n\n```ini\n[metadata]\nname = dark\ndescription = Dark mode theme\ntags = dark\ninherit = True\n\n[styles]\ndanger = bold red\ninfo = dim cyan\nwarning = bold magenta\n```\n\nHere\'s an real world example of a theme INI file from one of my [apps](https://github.com/RhetTbull/osxphotos):\n\n```INI\n[metadata]\nname = dark\ndescription = Dark mode theme\ntags = dark\ninherit = True\n\n[styles]\nbar.back = rgb(68,71,90)\nbar.complete = rgb(249,38,114)\nbar.finished = rgb(80,250,123)\nbar.pulse = rgb(98,114,164)\ncolor = rgb(248,248,242)\ncount = rgb(139,233,253)\nerror = bold rgb(255,85,85)\nfilename = bold rgb(189,147,249)\nfilepath = bold rgb(80,250,123)\nhighlight = bold #000000 on #d73a49\nnum = bold rgb(139,233,253)\nprogress.elapsed = rgb(139,233,253)\nprogress.percentage = rgb(255,121,198)\nprogress.remaining = rgb(139,233,253)\ntime = bold rgb(139,233,253)\nuuid = rgb(255,184,108)\nwarning = bold rgb(241,250,140)\n```\n\n`Theme` implements the [rich Console protocol](https://rich.readthedocs.io/en/stable/protocol.html) which means that you use `rich.print()` and `rich.console.Console().print()` to print a theme to the console.  Doing so results in a preview of the theme which visually shows the colors and styles used in the theme.\n\n![Theme preview](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/theme_preview_print.png)\n\nThe `Theme` preview will use default sample text for each style. You can change the sample text by setting the `rich_theme_manager.theme.SAMPLE_TEXT` global variable.\n\n![Theme preview with sample text](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/theme_preview_sample_text.png)\n\n`Theme` implements the `__eq__` method so two `Theme` instances can be easily compared for equality. `Theme` instances are considered equal if all properties with exception of `path` are equal.\n\n### ThemeManager class\n\n```python\n    ThemeManager(\n        theme_dir: Optional[str] = None,\n        themes: Optional[List[Theme]] = None,\n    )\n```\n\nArguments:\n\n* `theme_dir`: Optional directory to save/load themes to/from.\n* `themes`: Optional list of Theme objects\n\nIf provided, `theme_dir` must exist.  If `theme_dir` is set (for example, using [click.get_app_dir](https://click.palletsprojects.com/en/8.0.x/api/?highlight=get_app_dir#click.get_app_dir)), upon initialization `ThemeManager` will save any default themes provided via `themes` to `theme_dir` and load any themes from `theme_dir`.  Theme files are standard INI files as created by [configparser](https://docs.python.org/3/library/configparser.html) and are named `<name>.theme` where `<name>` is the name of the Theme (see `Theme.name`).  If a theme file already exists, it will be loaded and `ThemeManager` will not overwrite it.\n\nProperties:\n\n* `ThemeManager().themes`: List of themes\n\nMethods:\n\n* `ThemeManager().add(theme: Theme, overwrite=False) -> None`: Add a theme to the list of managed themes.  If `overwrite` is True, the theme file will be overwritten if it already exists.\n* `ThemeManager().remove(theme: Theme) -> None`: Remove a theme from the list of managed themes and delete the theme file if it exists.\n* `ThemeManager().get(theme_name: str) -> Theme`: Get a theme by name. Raises \'ValueError` if no theme with the given name is found.\n* `ThemeManager().load_themes(theme_dir=None) -> None`: Load themes from `theme_dir` (or `ThemeManager().theme_dir` if not provided).  Any `.theme` files found in `theme_dir` will be loaded and added to the list of managed themes.\n* `ThemeManager().write_themes(overwrite=False) -> None`: Write themes to file (as specified in each `Theme().path` which will be set automatically by `ThemeManager`).  If `overwrite` is True, the theme file will be overwritten if it already exists.\n\n* `ThemeManager().list_themes(show_path: bool = True, theme_names: Optional[List[str]] = None, console: Optional[Console] = None) -> None`: Print a list of themes to the console.  If `show_path` is True, the path to the theme file will be printed.  If `theme_names` is provided, only themes with names in the list will be printed. An optional `rich.console.Console()` instance may be provided to print to a specific console.\n\n![ThemeManager list example](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/theme_manager_list.png)\n\nClass Methods:\n\n* `ThemeManager.preview_theme(theme: Theme, sample_text: Optional[str] = None, show_path: bool = True, console: Optional[Console] = None) -> None`: Print a preview of the theme to the console showing the style of each style in the theme.  If `sample_text` is provided, it will be used as the sample text to preview otherwise a default string will be used.  If `show_path` is True, the path to the theme file will be printed.  An optional `rich.console.Console()` instance may be provided to print to a specific console.\n\n![ThemeManager preview example](https://github.com/RhetTbull/rich_theme_manager/raw/main/images/theme_manager_preview.png)\n\n## Test Coverage\n\n100% coverage of all code with exception of the example CLI app.\n\n```text\nName                               Stmts   Miss  Cover\n------------------------------------------------------\nrich_theme_manager/__init__.py         5      0   100%\nrich_theme_manager/manager.py         71      0   100%\nrich_theme_manager/theme.py          134      0   100%\ntests/__init__.py                      0      0   100%\ntests/conftest.py                      7      0   100%\ntests/test_rich_theme_manager.py     256      0   100%\n------------------------------------------------------\nTOTAL                                473      0   100%\n```\n\n## License\n\nMIT License\n\n## Contributing\n\nContributions of all kinds are welcome!  Please submit pull requests, issues, and/or suggestions to the [github repo](https://github.com/RhetTbull/rich_theme_manager).\n\n## Credits\n\nThank you to [Will McGugan](https://github.com/willmcgugan) for creating [rich](https://github.com/Textualize/rich) and helping to make our command line interfaces more beautiful!\n\n## Projects Using Rich Theme Manager\n\n* [osxphotos](https://github.com/RhetTbull/osxphotos): Python app to export pictures and associated metadata from Apple Photos on macOS. Also includes a package to provide programmatic access to the Photos library, pictures, and metadata.\n',
    'author': 'Rhet Turnbull',
    'author_email': 'rturnbull+git@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/RhetTbull/rich_theme_manager',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0.0',
}


setup(**setup_kwargs)
